﻿Imports ThinkGeo.MapSuite.Core
Imports ThinkGeo.MapSuite.WpfDesktopEdition

''' <summary>
''' This class is used to handle dismissing the trackshape.  It also has a property to determine if you are in a trackshape operation.  The Interactive click interval is changed because the is a discrepency between when the user clicks and the point is registerd when the vlaue is left at the default 500.
''' </summary>

Public Class CustomTrackInteractiveOverlay
  Inherits TrackInteractiveOverlay
  Private m_isTracking As Boolean = False

  Public Sub New()
    MyBase.New()

    'Setting the interactiveclick interval here ensures the first point is set in the proper location.  After the trackshape is started, the value is set to 400 in the customtrackoverlay.
    InteractiveClickInterval = 1

  End Sub

  Public ReadOnly Property IsTracking() As Boolean
    Get
      Return m_isTracking
    End Get
  End Property

  Public Shadows Sub EndTracking()
    MyBase.EndTracking()
  End Sub

  Protected Overrides Function MouseDownCore(ByVal interactionArguments As InteractionArguments) As InteractiveResult

    If interactionArguments.MouseButton <> MapMouseButton.Right Then
      Return MyBase.MouseDownCore(interactionArguments)
    Else
      RemoveLastVertexAdded()
    End If
    Return New InteractiveResult()

  End Function

  Public Sub RemoveLastVertexAdded()

    'Tries to remove the last vertex first
    Dim minVertex As Integer = 2
    If Me.Vertices.Count > minVertex Then
      SyncLock Me
        Dim lastVertex As Vertex = Me.Vertices(Me.Vertices.Count - 2)
        Me.Vertices.Remove(lastVertex)
      End SyncLock

      MyBase.Refresh()
    Else 'If only two or less vertex, remove the whole Feature
      Call RemoveLastFeatureAdded()
    End If

  End Sub

  Public Sub RemoveLastFeatureAdded()

    Dim count As Integer = Me.TrackShapeLayer.InternalFeatures.Count
    If count > 0 Then
      SyncLock Me
        Me.TrackShapeLayer.InternalFeatures.RemoveAt(count - 1)
      End SyncLock

      'Ends track mode
      Me.EndTracking()
      Me.MouseDoubleClick(New InteractionArguments())

      MyBase.Refresh()
    End If

  End Sub

  Protected Overrides Sub OnTrackStarted(ByVal e As TrackStartedTrackInteractiveOverlayEventArgs)
    InteractiveClickInterval = 400
    m_isTracking = True
    Mouse.OverrideCursor = Cursors.Cross
    MyBase.OnTrackStarted(e)
  End Sub

  Protected Overrides Sub OnTrackEnded(ByVal e As TrackEndedTrackInteractiveOverlayEventArgs)
    InteractiveClickInterval = 1
    m_isTracking = False
    Me.TrackMode = TrackMode.None
    Mouse.OverrideCursor = Cursors.Arrow
    MyBase.OnTrackEnded(e)
  End Sub

End Class
