﻿Imports System.Collections.ObjectModel
Imports ThinkGeo.MapSuite.Core
Imports ThinkGeo.MapSuite.WpfDesktopEdition

Public Class TrackAndEditShapes
  Private Sub WpfMap_Loaded(ByVal sender As Object, ByVal e As RoutedEventArgs)
    'Basic Map Settings
    wpfMap1.MapUnit = GeographyUnit.DecimalDegree
    wpfMap1.CurrentExtent = New RectangleShape(-360, 90, 360, -90)

    'Adds the background WorldMapKit
    Dim worldMapKitOverlay As New WorldMapKitWmsWpfOverlay()
    worldMapKitOverlay.WrappingMode = WrappingMode.WrapDateline
    wpfMap1.Overlays.Add(worldMapKitOverlay)

    'Creates the CustomTrackOverlay
    wpfMap1.TrackOverlay = New CustomTrackInteractiveOverlay()

    wpfMap1.Refresh()
  End Sub

  Private Sub button_Click(ByVal sender As Object, ByVal e As RoutedEventArgs)
    Dim button As Button = TryCast(sender, Button)
    If button IsNot Nothing Then
      Select Case button.Name
        Case "btnTrackNormal"
          wpfMap1.TrackOverlay.TrackMode = TrackMode.None
        Case "btnTrackLine"
          wpfMap1.TrackOverlay.TrackMode = TrackMode.Line
        Case "btnTrackSquare"
          wpfMap1.TrackOverlay.TrackMode = TrackMode.Square
        Case "btnTrackEdit"
          wpfMap1.TrackOverlay.TrackMode = TrackMode.None
          For Each feature As Feature In wpfMap1.TrackOverlay.TrackShapeLayer.InternalFeatures
            wpfMap1.EditOverlay.EditShapesLayer.InternalFeatures.Add(feature)
          Next

          wpfMap1.TrackOverlay.TrackShapeLayer.InternalFeatures.Clear()
          wpfMap1.EditOverlay.CalculateAllControlPoints()
          wpfMap1.Refresh(New Overlay() {wpfMap1.EditOverlay, wpfMap1.TrackOverlay})
          Exit Select
        Case "btnTrackDelete"
          ' should use TrackMode.EditShape select a shape, and then call DeleteTrackShape();
          Dim lastIndex As Integer = wpfMap1.EditOverlay.EditShapesLayer.InternalFeatures.Count - 1

          If lastIndex >= 0 Then
            wpfMap1.EditOverlay.EditShapesLayer.InternalFeatures.RemoveAt(lastIndex)
            wpfMap1.EditOverlay.CalculateAllControlPoints()
          End If

          wpfMap1.Refresh(wpfMap1.EditOverlay)
        Case Else
          wpfMap1.TrackOverlay.TrackMode = TrackMode.None
      End Select
    End If
  End Sub

#Region "Mouse Events"
  Dim ScreenMouseX As Integer = 0
  Dim ScreenMouseY As Integer = 0

  Private Sub WpfMap1_MouseMove(ByVal sender As Object, ByVal e As System.Windows.Input.MouseEventArgs) Handles wpfMap1.MouseMove
    Dim p As System.Windows.Point = e.GetPosition(Me)
    ScreenMouseX = p.X
    ScreenMouseY = p.Y
  End Sub

  Private Sub WpfMap1_MouseLeftButtonUp(ByVal sender As Object, ByVal e As System.Windows.Input.MouseButtonEventArgs) Handles wpfMap1.MouseLeftButtonUp
    '***** Standard mouse click (Selecting)
    Dim RefreshInteractiveOverlays As Boolean = False

    If wpfMap1.TrackOverlay.TrackMode <> TrackMode.None Then Exit Sub

    'Exit edit mode if blank zone clicked (unmark selected), or before selecting other
    If (Not wpfMap1.EditOverlay.IsEmpty) Then

      'Move the features from Edit to Track Overlay
      For Each feature As Feature In wpfMap1.EditOverlay.EditShapesLayer.InternalFeatures
        wpfMap1.TrackOverlay.TrackShapeLayer.InternalFeatures.Add(feature)
      Next

      'Clear Edit Overlay
      wpfMap1.EditOverlay.EditShapesLayer.InternalFeatures.Clear()
      wpfMap1.EditOverlay.CalculateAllControlPoints()

      RefreshInteractiveOverlays = True
    End If

    'Search for Features in the mouse position
    Dim SelectingFeatures As Collection(Of NearFeaturesHelper) = GetFeaturesOnScreenPos(New ScreenPointF(ScreenMouseX, ScreenMouseY))

    If SelectingFeatures.Count > 0 Then 'Any selected
      'Find the first visible feature
      For k As Integer = 0 To SelectingFeatures.Count - 1

        Select Case SelectingFeatures(k).FeatureType
          Case NearFeaturesHelper.FeatureTypes.MeasureLine
            wpfMap1.TrackOverlay.TrackMode = TrackMode.None

            'Move the feature from Track to Edit Overlay
            wpfMap1.EditOverlay.EditShapesLayer.InternalFeatures.Add(SelectingFeatures(k).Feature)
            wpfMap1.TrackOverlay.TrackShapeLayer.InternalFeatures.Remove(SelectingFeatures(k).Feature)

            wpfMap1.EditOverlay.CalculateAllControlPoints()
            RefreshInteractiveOverlays = True
            Exit For
        End Select
      Next
    End If

    'Refresh interactive overlays
    If RefreshInteractiveOverlays Then wpfMap1.Refresh(New Overlay() {wpfMap1.EditOverlay, wpfMap1.TrackOverlay})

  End Sub

#Region "Class NearFeaturesHelper"
  Private Class NearFeaturesHelper
    Public Feature As Feature
    Public FeatureType As FeatureTypes

    Public Enum FeatureTypes
      MeasureLine
      MeasureLineControlPoint
    End Enum

    Public Sub New(ByVal FeatureType As FeatureTypes, ByVal Feature As Feature)
      Me.FeatureType = FeatureType
      Me.Feature = Feature
    End Sub
  End Class
#End Region

  Private Function GetFeaturesOnScreenPos(ByVal ScreenPos As ScreenPointF) As Collection(Of NearFeaturesHelper)
    Dim ScreenBufferSize As Integer = 5
    Dim selectedFeatures As New Collection(Of NearFeaturesHelper)

    Try
      'Logic for converting screen coordinate values to world coordinate for the spatial query. Notice that the distance buffer for the spatial query
      'will change according to the zoom level while it remains the same for the screen buffer distance.
      Dim WorldPos As PointShape = ExtentHelper.ToWorldCoordinate(wpfMap1.CurrentExtent, ScreenPos.X, ScreenPos.Y, wpfMap1.ActualWidth, wpfMap1.ActualHeight)
      Dim bufferPointF As ScreenPointF = New ScreenPointF(ScreenPos.X + ScreenBufferSize, ScreenPos.Y)
      Dim correctedExtent As RectangleShape = wpfMap1.CurrentExtent

      'Gets the real (central) valid world coordinate. The "elegant" way would be by ussing a projection, but this is sorter just for testing
      If WorldPos.X < -180 Then
        'Calcs the valid longitude
        Do
          WorldPos.X += 360
        Loop While WorldPos.X < -180
        'Gets a corrected extent as well for further spatial calculations (within the "real" (center) world)
        Do
          correctedExtent.UpperLeftPoint.X += 360
          correctedExtent.LowerRightPoint.X += 360
        Loop While correctedExtent.UpperLeftPoint.X < -180
      ElseIf WorldPos.X > 180 Then
        'Calcs the valid longitude
        Do
          WorldPos.X -= 360
        Loop While WorldPos.X > 180
        'Gets a corrected extent as well for further spatial calculations (within the "real" (center) world)
        Do
          correctedExtent.UpperLeftPoint.X -= 360
          correctedExtent.LowerRightPoint.X -= 360
        Loop While correctedExtent.UpperLeftPoint.X > 180
      End If

      'Calcs the distanceBuffer cheating the GetWorldDistanceBetweenTwoScreenPoints as if map were in meters
      Dim distanceBuffer As Double = ExtentHelper.GetWorldDistanceBetweenTwoScreenPoints(correctedExtent, ScreenPos, bufferPointF, wpfMap1.ActualWidth, wpfMap1.ActualHeight, GeographyUnit.Meter, DistanceUnit.Meter)

      Dim columnNames As Collection(Of String) = New Collection(Of String)

      'Search in all the required layers the features within the distance range

      'Gets the Features on the EditOverlay
      wpfMap1.EditOverlay.ExistingControlPointsLayer.Open()
      'GetFeaturesNearesTo is also cheated as if FeatureSource were in meters
      For Each tmpFeature As Feature In wpfMap1.EditOverlay.ExistingControlPointsLayer.FeatureSource.GetFeaturesNearestTo(WorldPos, GeographyUnit.Meter, 5, columnNames, distanceBuffer, DistanceUnit.Meter)
        selectedFeatures.Insert(0, New NearFeaturesHelper(NearFeaturesHelper.FeatureTypes.MeasureLineControlPoint, tmpFeature)) 'Adds the feature to the collection
      Next
      'wpfMap1.EditOverlay.ExistingControlPointsLayer.Close 

      'Gets the Features on the TrackOverlay
      wpfMap1.TrackOverlay.TrackShapeLayer.Open()
      'GetFeaturesNearesTo is also cheated as if FeatureSource were in meters
      For Each tmpFeature As Feature In wpfMap1.TrackOverlay.TrackShapeLayer.FeatureSource.GetFeaturesNearestTo(WorldPos, GeographyUnit.Meter, 5, columnNames, distanceBuffer, DistanceUnit.Meter)
        selectedFeatures.Insert(0, New NearFeaturesHelper(NearFeaturesHelper.FeatureTypes.MeasureLine, tmpFeature)) 'Adds the feature to the collection
      Next
      'WpfMap1.TrackOverlay.TrackShapeLayer.Close()

    Catch ex As Exception
    End Try

    Return selectedFeatures
  End Function
#End Region
End Class
